
import 'dart:typed_data';


import 'package:chainwallet/src/core/serialized/enums/TransactionType.dart';
import 'package:chainwallet/src/utils/Util.dart';
import 'package:pointycastle/digests/sha256.dart';
import 'package:pointycastle/digests/ripemd160.dart';

import 'ecdsa/IkeyPair.dart';
import 'ecdsa/Seed.dart';
import 'package:chainwallet/src/utils/B58IdentiferCodecs.dart';

import 'package:chainwallet/src/core/types/tx/Transaction.dart';
import 'package:chainwallet/src/core/types/signed/SignedTransaction.dart';

class Chainsql  {


  /**
   * Generate a new account.
   *
   * @return Contains folling fields:
   * secret:Account secret.
   * address:Account address.
   * publicKey:Account publickey.
   */
  static Map generateAddress() {
   // Security.addProvider(new BouncyCastleProvider());
    Seed seed = Seed.randomSeed();
    return generateAddressFromSeed(seed);
  }

  static Map generateAddressFromStr(String secret) {

    Seed seed = Seed.fromBase58(secret);
    return generateAddressFromSeed(seed);
  }

  static Map generateAddressFromSeed(Seed seed) {

    IKeyPair keyPair = seed.keyPair();
    Uint8List pubBytes = keyPair.canonicalPubBytes();

    SHA256Digest sha = new SHA256Digest();
    sha.update(pubBytes, 0, pubBytes.length);

    var result = new Uint8List(sha.digestSize);
    sha.doFinal(result, 0);

    RIPEMD160Digest d = new RIPEMD160Digest();
    d.update(result, 0, result.length);
    var o = new Uint8List(d.digestSize);
    d.doFinal(o, 0);


    String publicKey = B58IdentiferCodecs.encode(pubBytes, B58IdentiferCodecs.VER_ACCOUNT_PUBLIC);
    String secretKey = B58IdentiferCodecs.encodeFamilySeed(seed.bytes());
    String address = B58IdentiferCodecs.encodeAddress(o);

    Map retMap = {"secret":secretKey,"address":address,"publicKey":publicKey};

    return retMap;
  }

  static Uint8List sign(Uint8List data,String secret){
//    for(int dat in data){
//      print(dat);
//    }
    Seed seed = Seed.fromBase58(secret);
    IKeyPair keyPair = seed.keyPair();
    return keyPair.signMessage(data);
  }


  static Map signTx(Map txMap, String secret) {

      if(txMap == null || !txMap.containsKey('tx_json')){

          Map ret = new Map();
          ret['error'] = "missing key 'tx_json'";
          return ret;
      }

      Map tx = txMap['tx_json'];
      if( !tx.containsKey('TransactionType')){

        Map ret = new Map();
        ret['error'] = "missing key 'TransactionType'";
        return ret;
      }


    TransactionType type = TransactionType.valueOf( tx['TransactionType']);
    Transaction transaction = new Transaction(type);

//      print("Transaction init0 \r\n");
//    print(Util.formatBytesAsHexString(transaction.toBytes()));

    transaction.parseFromJson(tx);

//    print("Transaction init1 \r\n");
//    print(Util.formatBytesAsHexString(transaction.toBytes()));
//
//    Transaction transaction1 = new Transaction(type);
//
//    print("Transaction init2 \r\n");
//    print(Util.formatBytesAsHexString(transaction1.toBytes()));


   SignedTransaction signed = transaction.sign(secret);

   Map ret = new Map();
  ret['tx_blob'] = signed.tx_blob;
  ret['hash']    = signed.hash.toString();





    return ret;
  }

}